// ShapeArcCreator.cpp: implementation of the CShapeArcCreator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ShapeArcCreator.h"
#include "shapeobjects.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//*******************************************************************************
// Name : CShapeArcCreator
// Desc : 
//*******************************************************************************
CShapeArcCreator::CShapeArcCreator(size_t nParts,
								   size_t nTotalPointCount,
								   size_t nRecordNumber)
{
	m_nParts = nParts;
	m_nTotalPointCount = nTotalPointCount;
	m_nRecordNumber = nRecordNumber + 1;
	m_pMem = NULL;
	m_pNextPoint = NULL;
	m_bPointWritten = false;
	m_nCurrentPart = 0;
}


//*******************************************************************************
// Name : ~CShapeArcCreator
// Desc : 
//*******************************************************************************
CShapeArcCreator::~CShapeArcCreator()
{

}


//*******************************************************************************
// Name : SetRecordNumber
// Desc : 
//*******************************************************************************
void CShapeArcCreator::SetRecordNumber(size_t nRecordNumber)
{
	m_nRecordNumber = nRecordNumber + 1;
}


//*******************************************************************************
// Name : SetNewRecordNumber
// Desc :
//*******************************************************************************
void CShapeArcCreator::SetNewRecordNumber(void* pMem, size_t nRecordNumber)
{
	SShapeArcRecord *s = reinterpret_cast<SShapeArcRecord*>(pMem);
	s->m_recordNumber = ByteSwitchInt(nRecordNumber + 1);
}


//*******************************************************************************
// Name : UpdateCoordinate
// Desc : 
//*******************************************************************************
void CShapeArcCreator::UpdateCoordinate(void* pMem, size_t nIndex, double x, double y)
{
	CShapeArcRecord *p = reinterpret_cast<CShapeArcRecord*>(pMem);
	CShapeArcRecord::XYPair& point = const_cast<CShapeArcRecord::XYPair&>(p->GetPoint(nIndex));
	point.x = x;
	point.y = y;

	// Update bounding box
	RecalcBoundingBox(pMem);
}


//*******************************************************************************
// Name : RecalcBoundingBox
// Desc : 
//*******************************************************************************
void CShapeArcCreator::RecalcBoundingBox(void* pMem)
{
	SShapeArcRecord *s = reinterpret_cast<SShapeArcRecord*>(pMem);
	CShapeArcRecord *p = reinterpret_cast<CShapeArcRecord*>(pMem);

	if (p->GetPointCount() > 0)
	{
		s->m_xMax = s->m_xMin = p->GetPoint(0).x;
		s->m_yMax = s->m_yMin = p->GetPoint(0).y;
	}
	else
		return;

	for (size_t i = 1; i < p->GetPointCount(); i++)
	{
		double x = p->GetPoint(i).x;
		double y = p->GetPoint(i).y;
		if (x < s->m_xMin)
			s->m_xMin = x;
		if (x > s->m_xMax)
			s->m_xMax = x;
		if (y < s->m_yMin)
			s->m_yMin = y;
		if (y > s->m_yMax)
			s->m_yMax = y;
	}
}


//*******************************************************************************
// Name : GetAllocSizeBytes
// Desc : 
//*******************************************************************************
size_t CShapeArcCreator::GetAllocSizeBytes() const
{
	return sizeof(SShapeArcRecord) +
			(m_nParts-1)*sizeof(int) + // -1, one int is already accounted for in SShapeArcRecord
			m_nTotalPointCount*sizeof(double)*2;
}


//*******************************************************************************
// Name : WriteHeader
// Desc : 
//*******************************************************************************
bool CShapeArcCreator::WriteHeader(void* pMem)
{
	m_pMem = reinterpret_cast<unsigned char*>(pMem);
	if (m_pMem != NULL)
	{
		SShapeArcRecord *s = reinterpret_cast<SShapeArcRecord*>(m_pMem);
		s->m_recordNumber = ByteSwitchInt(m_nRecordNumber);
		s->m_contentLength = ByteSwitchInt(GetAllocSizeBytes() - sizeof(int)*2); // Remove record header length
		s->IntShapeType = Shape_Arc;
		s->m_nPartCount = m_nParts;
		s->m_nPointCount = m_nTotalPointCount;
		s->m_parts[0] = 0;
		m_nCurrentPart = 0;
		m_bPointWritten = false;
		m_nSuccessfullyWrittenPoints = 0;
		m_pNextPoint = reinterpret_cast<unsigned char*>(&s->m_parts + m_nParts);

		return true;
	}
	return false;
}


//*******************************************************************************
// Name : WritePoint
// Desc : 
//*******************************************************************************
bool CShapeArcCreator::WritePoint(double x, double y)
{
	if (m_pMem != NULL && m_pNextPoint != NULL)
	{
		if (m_pNextPoint - m_pMem + sizeof(double)*2 > GetAllocSizeBytes())
			return false;

		m_nSuccessfullyWrittenPoints++;

		SShapeArcRecord *s = reinterpret_cast<SShapeArcRecord*>(m_pMem);
		if (m_bPointWritten)
		{
			if (x < s->m_xMin)
				s->m_xMin = x;
			if (x > s->m_xMax)
				s->m_xMax = x;
			if (y < s->m_yMin)
				s->m_yMin = y;
			if (y > s->m_yMax)
				s->m_yMax = y;
		}
		else
		{
			s->m_xMin = s->m_xMax = x;
			s->m_yMin = s->m_yMax = y;
			m_bPointWritten = true;
		}
		::memcpy(m_pNextPoint, &x, sizeof(double));
		m_pNextPoint += sizeof(double);
		::memcpy(m_pNextPoint, &y, sizeof(double));
		m_pNextPoint += sizeof(double);

		return true;
	}
	return false;
}


//*******************************************************************************
// Name : WritePoint
// Desc : 
//*******************************************************************************
bool CShapeArcCreator::WriteNewPart()
{
	if (m_pMem == NULL)
		return false;

	m_nCurrentPart++;

	if (m_nCurrentPart >= m_nParts)
		return false;

	SShapeArcRecord *s = reinterpret_cast<SShapeArcRecord*>(m_pMem);
	s->m_parts[m_nCurrentPart] = m_nSuccessfullyWrittenPoints;

	return true;
}

